@@ -1,22 +1,28 @@
 /*
  * ark -- archiver for the KDE project
  *
- * Copyright (C) 2008 Harald Hvaal <haraldhv (at@at) stud.ntnu.no>
+ * Copyright (C) 2008 Harald Hvaal <haraldhv@stud.ntnu.no>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ( INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION ) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * ( INCLUDING NEGLIGENCE OR OTHERWISE ) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "batchextract.h"
@@ -26,13 +32,6 @@
 #include "jobs.h"
 #include "queries.h"
 
-#include <QFileInfo>
-#include <QDir>
-#include <QCoreApplication>
-#include <QApplication>
-#include <QDesktopWidget>
-#include <QPointer>
-
 #include <KDebug>
 #include <KGlobal>
 #include <KLocale>
@@ -41,14 +40,21 @@
 #include <KIO/RenameDialog>
 #include <kwidgetjobtracker.h>
 
+#include <QDir>
+#include <QFileInfo>
+#include <QPointer>
+#include <QTimer>
+
 namespace Kerfuffle
 {
 BatchExtract::BatchExtract()
-        : m_autoSubfolders(false),
+        : m_autoSubfolder(false),
         m_preservePaths(true),
         m_openDestinationAfterExtraction(false)
 {
     setCapabilities(KJob::Killable);
+
+    connect(this, SIGNAL(result(KJob*)), SLOT(showFailedFiles()));
 }
 
 BatchExtract::~BatchExtract()
@@ -58,44 +64,36 @@ BatchExtract::~BatchExtract()
     }
 }
 
-void BatchExtract::addExtraction(Kerfuffle::Archive* archive, bool preservePaths, QString destinationFolder)
+void BatchExtract::addExtraction(Kerfuffle::Archive* archive)
 {
-    kDebug(1601);
+    QString destination = destinationFolder();
 
-    QString autoDestination = destinationFolder;
+    if ((autoSubfolder()) && (!archive->isSingleFolderArchive())) {
+        const QDir d(destination);
+        QString subfolderName = archive->subfolderName();
 
-    if (m_autoSubfolders) {
-        if (!archive->isSingleFolderArchive()) {
-            QDir destinationDir(destinationFolder);
-            QString subfolderName = archive->subfolderName();
+        if (d.exists(subfolderName))
+            subfolderName = KIO::RenameDialog::suggestName(destination, subfolderName);
 
-            if (destinationDir.exists(subfolderName))
-                subfolderName = KIO::RenameDialog::suggestName(destinationFolder, subfolderName);
+        d.mkdir(subfolderName);
 
-            kDebug(1601) << "Auto-creating subfolder" << subfolderName << "under" << destinationFolder;
-            destinationDir.mkdir(subfolderName);
-
-            autoDestination = destinationFolder + '/' + subfolderName;
-        }
+        destination += '/' + subfolderName;
     }
 
     Kerfuffle::ExtractionOptions options;
-    options["PreservePaths"] = preservePaths;
+    options["PreservePaths"] = preservePaths();
 
-    Kerfuffle::ExtractJob *job = archive->copyFiles(
-                                     QVariantList(), //extract all files
-                                     autoDestination, //extract to folder
-                                     options
-                                 );
+    Kerfuffle::ExtractJob *job = archive->copyFiles(QVariantList(), destination, options);
 
-    connect(job, SIGNAL(userQuery(Query*)), this, SLOT(slotUserQuery(Query*)));
-
-    kDebug(1601) << QString("Registering job from archive %1, to %2, preservePaths %3").arg(archive->fileName()).arg(autoDestination).arg(preservePaths);
+    kDebug() << QString("Registering job from archive %1, to %2, preservePaths %3").arg(archive->fileName()).arg(destination).arg(preservePaths());
 
     addSubjob(job);
-    m_fileNames[job] = qMakePair(archive->fileName(), destinationFolder);
+
+    m_fileNames[job] = qMakePair(archive->fileName(), destination);
+
     connect(job, SIGNAL(percent(KJob*, unsigned long)),
             this, SLOT(forwardProgress(KJob *, unsigned long)));
+    connect(job, SIGNAL(userQuery(Query*)), this, SLOT(slotUserQuery(Query*)));
 }
 
 void BatchExtract::slotUserQuery(Query *query)
@@ -103,12 +101,22 @@ void BatchExtract::slotUserQuery(Query *query)
     query->execute();
 }
 
+bool BatchExtract::autoSubfolder() const
+{
+    return m_autoSubfolder;
+}
+
 void BatchExtract::setAutoSubfolder(bool value)
 {
-    m_autoSubfolders = value;
+    m_autoSubfolder = value;
 }
 
 void BatchExtract::start()
+{
+    QTimer::singleShot(0, this, SLOT(slotStartJob()));
+}
+
+void BatchExtract::slotStartJob()
 {
     // If none of the archives could be loaded, there is no subjob to run
     if (m_inputs.isEmpty()) {
@@ -117,7 +125,7 @@ void BatchExtract::start()
     }
 
     foreach(Kerfuffle::Archive *archive, m_inputs) {
-        addExtraction(archive, m_preservePaths, destinationFolder());
+        addExtraction(archive);
     }
 
     KIO::getJobTracker()->registerJob(this);
@@ -130,16 +138,23 @@ void BatchExtract::start()
 
     m_initialJobCount = subjobs().size();
 
-    kDebug(1601) << "Starting first job";
+    kDebug() << "Starting first job";
 
     subjobs().at(0)->start();
 }
 
+void BatchExtract::showFailedFiles()
+{
+    if (!m_failedFiles.isEmpty()) {
+        KMessageBox::informationList(0, i18n("The following files could not be extracted:"), m_failedFiles);
+    }
+}
+
 void BatchExtract::slotResult(KJob *job)
 {
-    kDebug(1601);
+    kDebug();
     if (job->error()) {
-        kDebug(1601) << "There was en error, " << job->errorText();
+        kDebug() << "There was en error, " << job->errorText();
 
         setErrorText(job->errorText());
         setError(job->error());
@@ -157,17 +172,17 @@ void BatchExtract::slotResult(KJob *job)
         removeSubjob(job);
     }
 
-    if (!subjobs().size()) {
+    if (!hasSubjobs()) {
         if (openDestinationAfterExtraction()) {
             KUrl destination(destinationFolder());
             destination.cleanPath();
             KRun::runUrl(destination, "inode/directory", 0);
         }
 
-        kDebug(1601) << "Finished, emitting the result";
+        kDebug() << "Finished, emitting the result";
         emitResult();
     } else {
-        kDebug(1601) << "Starting the next job";
+        kDebug() << "Starting the next job";
         emit description(this,
                          i18n("Extracting file..."),
                          qMakePair(i18n("Source archive"), m_fileNames.value(subjobs().at(0)).first),
@@ -179,31 +194,36 @@ void BatchExtract::slotResult(KJob *job)
 
 void BatchExtract::forwardProgress(KJob *job, unsigned long percent)
 {
-    Q_UNUSED(job);
+    Q_UNUSED(job)
     int jobPart = 100 / m_initialJobCount;
     setPercent(jobPart *(m_initialJobCount - subjobs().size()) + percent / m_initialJobCount);
 }
 
 bool BatchExtract::addInput(const KUrl& url)
 {
-    kDebug(1601);
-
     Kerfuffle::Archive *archive = Kerfuffle::factory(url.pathOrUrl());
 
     if ((archive == NULL) || (!QFileInfo(url.pathOrUrl()).exists())) {
+        m_failedFiles.append(url.fileName());
         return false;
     }
 
-    m_inputs << archive;
+    m_inputs.append(archive);
+
     return true;
 }
 
-bool BatchExtract::openDestinationAfterExtraction()
+bool BatchExtract::openDestinationAfterExtraction() const
 {
     return m_openDestinationAfterExtraction;
 }
 
-QString BatchExtract::destinationFolder()
+bool BatchExtract::preservePaths() const
+{
+    return m_preservePaths;
+}
+
+QString BatchExtract::destinationFolder() const
 {
     if (m_destinationFolder.isEmpty()) {
         return QDir::currentPath();
@@ -212,7 +232,7 @@ QString BatchExtract::destinationFolder()
     }
 }
 
-void BatchExtract::setDestinationFolder(QString folder)
+void BatchExtract::setDestinationFolder(const QString& folder)
 {
     if (QFileInfo(folder).isDir()) {
         m_destinationFolder = folder;
@@ -231,16 +251,15 @@ void BatchExtract::setPreservePaths(bool value)
 
 bool BatchExtract::showExtractDialog()
 {
-    kDebug(1601);
-
     QPointer<Kerfuffle::ExtractionDialog> dialog = new Kerfuffle::ExtractionDialog(NULL);
+
     if (m_inputs.size() > 1) {
         dialog->batchModeOption();
     }
 
-    dialog->setAutoSubfolder(m_autoSubfolders);
+    dialog->setAutoSubfolder(autoSubfolder());
     dialog->setCurrentUrl(destinationFolder());
-    dialog->setPreservePaths(m_preservePaths);
+    dialog->setPreservePaths(preservePaths());
 
     if (m_inputs.size() == 1) {
         if (m_inputs.at(0)->isSingleFolderArchive()) {
