@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (C) 2002 Jean-Baptiste Mardelle <bj@altern.org>
  * Copyright (C) 2007,2008 Rolf Eike Beer <kde@opensource.sf-tec.de>
  */
@@ -22,6 +22,9 @@ class GroupEditProxyModel;
 class KGpgNode;
 class KGpgItemModel;
 
+/**
+ * @brief shows a widget that let's you change the keys that are part of a key group
+ */
 class groupEdit : public QWidget, public Ui::groupEdit
 {
 	Q_OBJECT
@@ -32,17 +35,41 @@ private:
 	KGpgItemModel *m_model;
 
 public:
-	QList<KGpgNode *> *members;
+	QList<KGpgNode *> *members;	///< the list of keys that are members of the group
 
+	/**
+	 * @brief constructor
+	 * @param parent parent widget
+	 * @param ids the members of the group
+	 */
 	explicit groupEdit(QWidget *parent, QList<KGpgNode *> *ids);
+	/**
+	 * @brief destructor
+	 */
 	~groupEdit();
 
+	/**
+	 * @brief set the model to select the keys from
+	 * @param md model to use
+	 */
 	void setModel(KGpgItemModel *md);
 
 private Q_SLOTS:
+	/**
+	 * @brief called when the add button is clicked
+	 */
 	void groupAdd();
+	/**
+	 * @brief called when the remove button is clicked
+	 */
 	void groupRemove();
+	/**
+	 * @brief called when an available key is double clicked
+	 */
 	void groupAdd(const QModelIndex &index);
+	/**
+	 * @brief clicked when a group member key is double clicked
+	 */
 	void groupRemove(const QModelIndex &index);
 };
 
