@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (C) 2002 Jean-Baptiste Mardelle <bj@altern.org>
  * Copyright (C) 2007 Jimmy Gilles <jimmygilles@gmail.com>
  * Copyright (C) 2008 Rolf Eike Beer <kde@opensource.sf-tec.de>
@@ -39,6 +39,8 @@
 #include "kgpgchangekey.h"
 #include "kgpgchangepass.h"
 #include "kgpgitemnode.h"
+#include "kgpgitemmodel.h"
+#include "selectexpirydate.h"
 
 using namespace KgpgCore;
 
@@ -95,75 +97,24 @@ void KgpgTrustLabel::change()
     m_color_w->setPalette(palette);
 }
 
-KgpgDateDialog::KgpgDateDialog(QWidget *parent, QDate date)
-              : KDialog(parent)
+KgpgKeyInfo::KgpgKeyInfo(KGpgKeyNode *node, KGpgItemModel *model, QWidget *parent)
+	: KDialog(parent),
+	keychange(new KGpgChangeKey(node)),
+	m_key(node->getKey()),
+	m_node(node),
+	m_model(model)
 {
-    setCaption(i18n("Choose New Expiration"));
-    setButtons(Ok | Cancel);
-    setDefaultButton(Ok);
-    setModal(true);
-
-    QWidget *page = new QWidget(this);
-    m_unlimited = new QCheckBox(i18nc("Key has unlimited lifetime", "Unlimited"), page);
-
-    if (date.isNull())
-        date = QDate::currentDate();
-
-    m_datepicker = new KDatePicker(date, page);
-    if (date.isNull()) {
-        m_datepicker->setEnabled(false);
-        m_unlimited->setChecked(true);
-    }
-
-    QVBoxLayout *layout = new QVBoxLayout(page);
-    layout->setSpacing(3);
-    layout->addWidget(m_datepicker);
-    layout->addWidget(m_unlimited);
-
-    connect(m_unlimited, SIGNAL(toggled(bool)), this, SLOT(slotEnableDate(bool)));
-    connect(m_datepicker, SIGNAL(dateChanged(QDate)), this, SLOT(slotCheckDate(QDate)));
-    connect(m_datepicker, SIGNAL(dateEntered(QDate)), this, SLOT(slotCheckDate(QDate)));
-
-    setMainWidget(page);
-    show();
-}
-
-QDate KgpgDateDialog::date() const
-{
-    if (m_unlimited->isChecked())
-        return QDate();
-    else
-        return m_datepicker->date();
-}
-
-void KgpgDateDialog::slotCheckDate(const QDate &date)
-{
-    enableButtonOk(date >= QDate::currentDate());
-}
-
-void KgpgDateDialog::slotEnableDate(const bool &ison)
-{
-    m_datepicker->setEnabled(!ison);
-    if (ison)
-        enableButtonOk(true);
-    else
-        slotCheckDate(m_datepicker->date());
-}
-
-KgpgKeyInfo::KgpgKeyInfo(KGpgKeyNode *node, QWidget *parent)
-           : KDialog(parent), keychange(new KGpgChangeKey(node))
-{
-    m_node = node;
-    m_key = node->getKey();
-    init();
+	init();
 }
 
 KgpgKeyInfo::KgpgKeyInfo(KgpgCore::KgpgKey *key, QWidget *parent)
-           : KDialog(parent), keychange(new KGpgChangeKey(key))
+	: KDialog(parent),
+	keychange(new KGpgChangeKey(key)),
+	m_key(key),
+	m_node(NULL),
+	m_model(NULL)
 {
-    m_node = NULL;
-    m_key = key;
-    init();
+	init();
 }
 
 void KgpgKeyInfo::init()
@@ -403,6 +354,28 @@ void KgpgKeyInfo::reloadKey()
 	displayKey();
 }
 
+void KgpgKeyInfo::reloadNode()
+{
+	Q_ASSERT(m_model != NULL);
+	Q_ASSERT(m_node != NULL);
+
+	const QString kid(m_node->getId());
+
+	// this will delete m_node
+	m_key = NULL;
+	m_model->refreshKey(m_node);
+
+	m_node = m_model->getRootNode()->findKey(kid);
+	if (m_node != NULL) {
+		m_key = m_node->getKey();
+		displayKey();
+	} else {
+		KMessageBox::error(this, i18n("<qt>The requested key is not present in the keyring anymore.<br />Perhaps it was deleted by another application</qt>"), i18n("Key not found"));
+		m_keywaschanged = false;
+		close();
+	}
+}
+
 void KgpgKeyInfo::displayKey()
 {
     QString name = m_key->name();
@@ -514,7 +487,7 @@ void KgpgKeyInfo::slotPreOk()
 
 void KgpgKeyInfo::slotChangeDate()
 {
-	KgpgDateDialog *dialog = new KgpgDateDialog(this, m_key->expirationDate());
+	QPointer<SelectExpiryDate> dialog = new SelectExpiryDate(this, m_key->expirationDate());
 	if (dialog->exec() == QDialog::Accepted) {
 		keychange->setExpiration(dialog->date());
 		enableButtonApply(keychange->wasChanged());
@@ -593,7 +566,10 @@ void KgpgKeyInfo::slotApplied(int result)
 		KMessageBox::error(this, i18n("Changing key properties failed."), i18n("Key properties"));
 	} else {
 		m_keywaschanged = true;
-		reloadKey();
+		if (m_node != NULL)
+			reloadNode();
+		else
+			reloadKey();
 	}
 	setControlEnable(true);
 }
