@@ -82,17 +82,6 @@ class kgpgOptions : public KConfigDialog
     Q_OBJECT
 
 public:
-    enum KeyColors
-    {
-        GoodColor = 0,
-        BadColor = 1,
-        UnknownColor = 2,
-        RevColor = 3,
-        UltimateColor = 4,
-        MarginalColor = 5,
-        ExpiredColor = 6
-    };
-
     explicit kgpgOptions(QWidget *parent = 0, const char *name = 0);
     ~kgpgOptions();
 
@@ -119,6 +108,7 @@ private slots:
     void slotInstallDecrypt(const QString &mimetype);
     void slotInstallSign(const QString &mimetype);
     void slotRemoveMenu(const QString &menu);
+    void slotSystrayEnable();
 
 private:
     bool hasChanged();
@@ -165,6 +155,7 @@ private:
     bool m_encrypttoalways;
     bool m_defaultencrypttoalways;
     bool m_showsystray;
+    int m_trayaction;
 };
 
 #endif // KGPGOPTIONS_H
